@@ -2,7 +2,7 @@
  * demux.c
  *****************************************************************************
  * Copyright (C) 2004, 2008-2009 VideoLAN
- * $Id: demux.c 69 2009-11-16 10:20:15Z md $
+ * $Id: demux.c 96 2010-01-06 16:20:38Z massiot $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *          Andy Gatward <a.j.gatward@reading.ac.uk>
@@ -117,7 +117,7 @@ void demux_Open( void )
 
     memset( p_pids, 0, sizeof(p_pids) );
 
-    dvb_Open();
+    pf_Open();
 
     for ( i = 0; i < 8192; i++ )
     {
@@ -126,7 +126,7 @@ void demux_Open( void )
     }
 
     if ( b_budget_mode )
-        i_demux_fd = dvb_SetFilter(8192);
+        i_demux_fd = pf_SetFilter(8192);
 
     SetPID(PAT_PID); /* PAT */
     p_pat_dvbpsi_handle = dvbpsi_AttachPAT( PATCallback, NULL );
@@ -148,7 +148,7 @@ void demux_Open( void )
  *****************************************************************************/
 void demux_Run( void )
 {
-    block_t *p_ts = dvb_Read();
+    block_t *p_ts = pf_Read();
 
     while ( p_ts != NULL )
     {
@@ -207,7 +207,7 @@ static void demux_Handle( block_t *p_ts )
         {
             for ( i = 0; i < i_nb_outputs; i++ )
             {
-                if ( pp_outputs[i]->i_maddr && pp_outputs[i]->p_sdt_section )
+                if ( ( pp_outputs[i]->i_config & OUTPUT_VALID ) && pp_outputs[i]->p_sdt_section )
                     output_Put( pp_outputs[i], p_ts );
             }
         }
@@ -255,12 +255,13 @@ static void demux_Handle( block_t *p_ts )
         output_t *p_output = p_pids[i_pid].pp_outputs[i];
         if ( p_output != NULL )
         {
-            if ( i_ca_handle && p_output->b_watch )
+            if ( i_ca_handle && (p_output->i_config & OUTPUT_WATCH) &&
+                 block_UnitStart( p_ts ) )
             {
                 uint8_t *p_payload;
 
                 if ( block_GetScrambling( p_ts ) ||
-                     ( block_UnitStart( p_ts ) && p_pids[i_pid].b_pes
+                     ( p_pids[i_pid].b_pes
                         && (p_payload = block_GetPayload( p_ts )) != NULL
                         && p_payload + 3 < p_ts->p_ts + TS_SIZE
                           && (p_payload[0] != 0 || p_payload[1] != 0
@@ -274,15 +275,13 @@ static void demux_Handle( block_t *p_ts )
 
                 if ( p_output->i_nb_errors > MAX_ERRORS )
                 {
-                    struct in_addr s;
                     int j;
                     for ( j = 0; j < i_nb_outputs; j++ )
                         pp_outputs[j]->i_nb_errors = 0;
 
-                    s.s_addr = p_output->i_maddr;
                     msg_Warn( NULL,
-                             "too many errors for stream %s:%d, resetting",
-                             inet_ntoa( s ), p_output->i_port );
+                             "too many errors for stream %s, resetting",
+                             p_output->psz_displayname );
                     en50221_Reset();
                 }
             }
@@ -291,7 +290,7 @@ static void demux_Handle( block_t *p_ts )
         }
     }
 
-    if ( output_dup.i_maddr )
+    if ( output_dup.i_config & OUTPUT_VALID )
         output_Put( &output_dup, p_ts );
 
     p_ts->i_refcount--;
@@ -442,7 +441,7 @@ static void SetPID( uint16_t i_pid )
 
     if ( !b_budget_mode && p_pids[i_pid].i_refcount
           && p_pids[i_pid].i_demux_fd == -1 )
-        p_pids[i_pid].i_demux_fd = dvb_SetFilter( i_pid );
+        p_pids[i_pid].i_demux_fd = pf_SetFilter( i_pid );
 }
 
 static void UnsetPID( uint16_t i_pid )
@@ -452,7 +451,7 @@ static void UnsetPID( uint16_t i_pid )
     if ( !b_budget_mode && !p_pids[i_pid].i_refcount
           && p_pids[i_pid].i_demux_fd != -1 )
     {
-        dvb_UnsetFilter( p_pids[i_pid].i_demux_fd, i_pid );
+        pf_UnsetFilter( p_pids[i_pid].i_demux_fd, i_pid );
         p_pids[i_pid].i_demux_fd = -1;
     }
 }
@@ -515,7 +514,7 @@ static void SelectPID( uint16_t i_sid, uint16_t i_pid )
     int i;
 
     for ( i = 0; i < i_nb_outputs; i++ )
-        if ( pp_outputs[i]->i_maddr && pp_outputs[i]->i_sid == i_sid
+        if ( ( pp_outputs[i]->i_config & OUTPUT_VALID ) && pp_outputs[i]->i_sid == i_sid
               && !pp_outputs[i]->i_nb_pids )
             StartPID( pp_outputs[i], i_pid );
 }
@@ -525,7 +524,7 @@ static void UnselectPID( uint16_t i_sid, uint16_t i_pid )
     int i;
 
     for ( i = 0; i < i_nb_outputs; i++ )
-        if ( pp_outputs[i]->i_maddr && pp_outputs[i]->i_sid == i_sid
+        if ( ( pp_outputs[i]->i_config & OUTPUT_VALID ) && pp_outputs[i]->i_sid == i_sid
               && !pp_outputs[i]->i_nb_pids )
             StopPID( pp_outputs[i], i_pid );
 }
@@ -538,7 +537,7 @@ static void SelectPSI( uint16_t i_sid, uint16_t i_pid )
     int i;
 
     for ( i = 0; i < i_nb_outputs; i++ )
-        if ( pp_outputs[i]->i_maddr && pp_outputs[i]->i_sid == i_sid )
+        if ( ( pp_outputs[i]->i_config & OUTPUT_VALID ) && pp_outputs[i]->i_sid == i_sid )
             SetPID( i_pid );
 }
 
@@ -547,7 +546,7 @@ static void UnselectPSI( uint16_t i_sid, uint16_t i_pid )
     int i;
 
     for ( i = 0; i < i_nb_outputs; i++ )
-        if ( pp_outputs[i]->i_maddr && pp_outputs[i]->i_sid == i_sid )
+        if ( ( pp_outputs[i]->i_config & OUTPUT_VALID ) && pp_outputs[i]->i_sid == i_sid )
             UnsetPID( i_pid );
 }
 
@@ -675,7 +674,7 @@ static void SendPAT( void )
 
     for ( i = 0; i < i_nb_outputs; i++ )
     {
-        if ( !pp_outputs[i]->i_maddr )
+        if ( !( pp_outputs[i]->i_config & OUTPUT_VALID ) )
             continue;
 
         if ( pp_outputs[i]->p_pat_section == NULL &&
@@ -720,7 +719,7 @@ static void SendSDT( void )
 
     for ( i = 0; i < i_nb_outputs; i++ )
     {
-        if ( pp_outputs[i]->i_maddr && pp_outputs[i]->p_sdt_section != NULL )
+        if ( ( pp_outputs[i]->i_config & OUTPUT_VALID ) && pp_outputs[i]->p_sdt_section != NULL )
         {
             block_t *p_block;
 
@@ -746,7 +745,7 @@ static void SendPMT( sid_t *p_sid )
 
     for ( i = 0; i < i_nb_outputs; i++ )
     {
-        if ( pp_outputs[i]->i_maddr && pp_outputs[i]->i_sid == p_sid->i_sid )
+        if ( ( pp_outputs[i]->i_config & OUTPUT_VALID ) && pp_outputs[i]->i_sid == p_sid->i_sid )
         {
             if ( pp_outputs[i]->p_pmt_section != NULL )
             {
@@ -778,7 +777,7 @@ static void SendEIT( dvbpsi_psi_section_t *p_section, uint16_t i_sid,
 
     for( i = 0; i < i_nb_outputs; i++ )
     {
-        if ( pp_outputs[i]->i_maddr && pp_outputs[i]->i_sid == i_sid )
+        if ( ( pp_outputs[i]->i_config & OUTPUT_VALID ) && pp_outputs[i]->i_sid == i_sid )
         {
             block_t *p_block;
 
@@ -959,7 +958,7 @@ static void UpdatePAT( uint16_t i_sid )
     int i;
 
     for ( i = 0; i < i_nb_outputs; i++ )
-        if ( pp_outputs[i]->i_maddr && pp_outputs[i]->i_sid == i_sid )
+        if ( ( pp_outputs[i]->i_config & OUTPUT_VALID ) && pp_outputs[i]->i_sid == i_sid )
             NewPAT( pp_outputs[i] );
 }
 
@@ -971,7 +970,7 @@ static void UpdatePMT( uint16_t i_sid )
     int i;
 
     for ( i = 0; i < i_nb_outputs; i++ )
-        if ( pp_outputs[i]->i_maddr && pp_outputs[i]->i_sid == i_sid )
+        if ( ( pp_outputs[i]->i_config & OUTPUT_VALID ) && pp_outputs[i]->i_sid == i_sid )
             NewPMT( pp_outputs[i] );
 }
 
@@ -983,7 +982,7 @@ static int SIDIsSelected( uint16_t i_sid )
     int i;
 
     for ( i = 0; i < i_nb_outputs; i++ )
-        if ( pp_outputs[i]->i_maddr && pp_outputs[i]->i_sid == i_sid )
+        if ( ( pp_outputs[i]->i_config & OUTPUT_VALID ) && pp_outputs[i]->i_sid == i_sid )
             return 1;
 
     return 0;
@@ -1412,7 +1411,7 @@ static void SDTCallback( void *_unused, dvbpsi_sdt_t *p_sdt )
 
     for ( i = 0; i < i_nb_outputs; i++ )
     {
-        if ( pp_outputs[i]->i_maddr ) 
+        if ( pp_outputs[i]->i_config & OUTPUT_VALID )
             NewSDT( pp_outputs[i] );
     }
 
