@@ -3,7 +3,7 @@
 # dvblast_mmi.sh
 ###############################################################################
 # Copyright (C) 1998-2008 VideoLAN
-# $Id: dvblast_mmi.sh 27 2009-10-20 19:15:04Z massiot $
+# $Id: dvblast_mmi.sh 105 2010-02-20 10:53:29Z md $
 #
 # Authors: Christophe Massiot <massiot@via.ecp.fr>
 #
@@ -124,6 +124,10 @@ while :; do
 		echo "CAUTION: the password won't be bulleted, be alone"
 		echo -n "Your choice (empty for extra choices) ? "
 		;;
+	252)
+		sleep 1
+		continue
+		;;
 	0)
 		echo -n "Your choice: (B)ack, (C)lose or (R)etry ? "
 		;;
