@@ -2,7 +2,7 @@
  * dvblastctl.c
  *****************************************************************************
  * Copyright (C) 2008 VideoLAN
- * $Id: dvblastctl.c 61 2009-11-07 11:16:00Z massiot $
+ * $Id: dvblastctl.c 105 2010-02-20 10:53:29Z md $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -33,15 +33,19 @@
 #include <arpa/inet.h>
 #include <sys/un.h>
 #include <errno.h>
+#include <getopt.h>
 
 #include "dvblast.h"
 #include "en50221.h"
 #include "comm.h"
+#include "version.h"
 
 int i_verbose = 3;
 
 void usage()
 {
+    msg_Raw( NULL, "DVBlastctl %d.%d.%d%s", VERSION_MAJOR, VERSION_MINOR, 
+					 VERSION_REVISION, VERSION_EXTRA );
     msg_Raw( NULL, "Usage: dvblastctl -r <remote socket> reload|shutdown|fe_status|mmi_status|mmi_open|mmi_close|mmi_get|mmi_send_text|mmi_send_choice [<CAM slot>] [<text/choice>]" );
     exit(1);
 }
@@ -60,7 +64,14 @@ int main( int i_argc, char **ppsz_argv )
     {
         int c;
 
-        if ( (c = getopt(i_argc, ppsz_argv, "r:h")) == -1 )
+        static const struct option long_options[] =
+        {
+            {"remote-socket", required_argument, NULL, 'r'},
+            {"help", no_argument, NULL, 'h'},
+            {0, 0, 0, 0}
+        };
+
+        if ( (c = getopt_long(i_argc, ppsz_argv, "r:h", long_options, NULL)) == -1 )
             break;
 
         switch ( c )
@@ -241,6 +252,10 @@ int main( int i_argc, char **ppsz_argv )
         exit(0);
         break;
 
+    case RET_MMI_WAIT:
+        exit(252);
+        break;
+
     case RET_ERR:
         msg_Err( NULL, "request failed" );
         exit(255);
